DEFAULT_PLUGINS = (
    'nose2.plugins.loader.discovery',
    'nose2.plugins.loader.testcases',
    'nose2.plugins.loader.functions',
    'nose2.plugins.loader.testclasses',
    'nose2.plugins.loader.generators',
    'nose2.plugins.loader.parameters',
    'nose2.plugins.loader.loadtests',
    'nose2.plugins.dundertest',
    'nose2.plugins.coverage',
    'nose2.plugins.result',
    'nose2.plugins.logcapture',
    'nose2.plugins.buffer',
    'nose2.plugins.failfast',
    'nose2.plugins.debugger',
    'nose2.plugins.prettyassert',
)
